@@ -2,26 +2,39 @@
 #  of resources by test cases.
 #
 #  Copyright (c) 2005-2010 Testresources Contributors
-#  
+#
 #  Licensed under either the Apache License, Version 2.0 or the BSD 3-clause
 #  license at the users choice. A copy of both licenses are available in the
 #  project source as Apache-2.0 and BSD. You may not use this file except in
 #  compliance with one of these two licences.
-#  
-#  Unless required by applicable law or agreed to in writing, software distributed
-#  under these licenses is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
-#  CONDITIONS OF ANY KIND, either express or implied.  See the license you chose
-#  for the specific language governing permissions and limitations under that
-#  license.
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under these licenses is distributed on an "AS IS" BASIS, WITHOUT
+#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
+#  license you chose for the specific language governing permissions and
+#  limitations under that license.
 #
 
 """TestResources: declarative management of external resources for tests."""
 
 import heapq
 import inspect
-import operator
 import unittest
 
+# same format as sys.version_info: "A tuple containing the five components of
+# the version number: major, minor, micro, releaselevel, and serial. All
+# values except releaselevel are integers; the release level is 'alpha',
+# 'beta', 'candidate', or 'final'. The version_info value corresponding to the
+# Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
+# releaselevel of 'dev' for unreleased under-development code.
+#
+# If the releaselevel is 'alpha' then the major/minor/micro components are not
+# established at this point, and setup.py will use a version of next-$(revno).
+# If the releaselevel is 'final', then the tarball will be major.minor.micro.
+# Otherwise it is major.minor.micro~$(revno).
+
+__version__ = (0, 2, 7, 'final', 0)
+
 
 def test_suite():
     import testresources.tests
@@ -41,12 +54,12 @@ def _digraph_to_graph(digraph, prime_node_mapping):
         No other edges are created.
     """
     result = {}
-    for from_node, from_prime_node in prime_node_mapping.iteritems():
-        result[from_node] = {from_prime_node:0}
-        result[from_prime_node] = {from_node:0}
-    for from_node, to_nodes in digraph.iteritems():
+    for from_node, from_prime_node in prime_node_mapping.items():
+        result[from_node] = {from_prime_node: 0}
+        result[from_prime_node] = {from_node: 0}
+    for from_node, to_nodes in digraph.items():
         from_prime = prime_node_mapping[from_node]
-        for to_node, value in to_nodes.iteritems():
+        for to_node, value in to_nodes.items():
             to_prime = prime_node_mapping[to_node]
             result[from_prime][to_node] = value
             result[to_node][from_prime] = value
@@ -67,13 +80,13 @@ def _kruskals_graph_MST(graph):
     forest = {}
     # graphs is the count of graphs we have yet to combine.
     for node in graph:
-        forest[node] = {node:{}}
+        forest[node] = {node: {}}
     graphs = len(forest)
     # collect edges: every edge is present twice (due to the graph
     # representation), so normalise.
     edges = set()
-    for from_node, to_nodes in graph.iteritems():
-        for to_node, value in to_nodes.iteritems():
+    for from_node, to_nodes in graph.items():
+        for to_node, value in to_nodes.items():
             edge = (value,) + tuple(sorted([from_node, to_node]))
             edges.add(edge)
     edges = list(edges)
@@ -84,10 +97,10 @@ def _kruskals_graph_MST(graph):
         g1 = forest[edge[1]]
         g2 = forest[edge[2]]
         if g1 is g2:
-            continue # already joined
+            continue  # already joined
         # combine g1 and g2 into g1
         graphs -= 1
-        for from_node, to_nodes in g2.iteritems():
+        for from_node, to_nodes in g2.items():
             #remember its symmetric, don't need to do 'to'.
             forest[from_node] = g1
             g1.setdefault(from_node, {}).update(to_nodes)
@@ -96,10 +109,10 @@ def _kruskals_graph_MST(graph):
         g1[edge[2]][edge[1]] = edge[0]
     # union the remaining graphs
     _, result = forest.popitem()
-    for _, g2 in forest.iteritems():
-        if g2 is result: # common case
+    for _, g2 in forest.items():
+        if g2 is result:  # common case
             continue
-        for from_node, to_nodes in g2.iteritems():
+        for from_node, to_nodes in g2.items():
             result.setdefault(from_node, {}).update(to_nodes)
     return result
 
@@ -120,7 +133,7 @@ def _resource_graph(resource_sets):
         for resource in resource_set:
             edges.setdefault(resource, []).append(node)
     # populate the adjacent members of nodes
-    for node, connected in nodes.iteritems():
+    for node, connected in nodes.items():
         for resource in node:
             connected.update(edges[resource])
         connected.discard(node)
@@ -138,7 +151,8 @@ def split_by_resources(tests):
     resource_set_tests = {no_resources: []}
     for test in tests:
         resources = getattr(test, "resources", ())
-        all_resources = list(resource.neededResources() for _, resource in  resources)
+        all_resources = list(resource.neededResources()
+                             for _, resource in resources)
         resource_set = set()
         for resource_list in all_resources:
             resource_set.update(resource_list)
@@ -148,7 +162,7 @@ def split_by_resources(tests):
 
 def _strongly_connected_components(graph, no_resources):
     """Find the strongly connected components in graph.
-    
+
     This is essentially a nonrecursive flatterning of Tarjan's method. It
     may be worth profiling against an actual Tarjan's implementation at some
     point, but sets are often faster than python calls.
@@ -190,7 +204,8 @@ class OptimisingTestSuite(unittest.TestSuite):
         except TypeError:
             unittest.TestSuite.addTest(self, test_case_or_suite)
             return
-        if test_case_or_suite.__class__ in (unittest.TestSuite, OptimisingTestSuite):
+        if test_case_or_suite.__class__ in (unittest.TestSuite,
+                                            OptimisingTestSuite):
             for test in tests:
                 self.adsorbSuite(test)
         else:
@@ -258,17 +273,17 @@ class OptimisingTestSuite(unittest.TestSuite):
         # We group the tests by the resource combinations they use,
         # since there will usually be fewer resource combinations than
         # actual tests and there can never be more: This gives us 'nodes' or
-        # 'resource_sets' that represent many tests using the same set of 
+        # 'resource_sets' that represent many tests using the same set of
         # resources.
         resource_set_tests = split_by_resources(self._tests)
         # Partition into separate sets of resources, there is no ordering
         # preference between sets that do not share members. Rationale:
         # If resource_set A and B have no common resources, AB and BA are
         # equally good - the global setup/teardown sums are identical. Secondly
-        # if A shares one or more resources with C, then pairing AC|CA is better
-        # than having B between A and C, because the shared resources can be
-        # reset or reused. Having partitioned we can use connected graph logic
-        # on each partition.
+        # if A shares one or more resources with C, then pairing AC|CA is
+        # better than having B between A and C, because the shared resources
+        # can be reset or reused. Having partitioned we can use connected graph
+        # logic on each partition.
         resource_set_graph = _resource_graph(resource_set_tests)
         no_resources = frozenset()
         # A list of resource_set_tests, all fully internally connected.
@@ -276,8 +291,8 @@ class OptimisingTestSuite(unittest.TestSuite):
             no_resources)
         result = []
         for partition in partitions:
-            # we process these at the end for no particularly good reason (it makes
-            # testing slightly easier).
+            # we process these at the end for no particularly good reason (it
+            # makes testing slightly easier).
             if partition == [no_resources]:
                 continue
             order = self._makeOrder(partition)
@@ -311,14 +326,14 @@ class OptimisingTestSuite(unittest.TestSuite):
                 from_resources = from_set
             for to_set in resource_sets:
                 if from_set is to_set:
-                    continue # no self-edges
+                    continue  # no self-edges
                 #if to_set == bottom:
                 #   if from_set == root:
                 #       continue # no short cuts!
                 #   to_resources = no_resources
                 #el
                 if to_set == root:
-                    continue # no links to root
+                    continue  # no links to root
                 else:
                     to_resources = to_set
                 graph[from_set][to_set] = self.cost_of_switching(
@@ -327,10 +342,12 @@ class OptimisingTestSuite(unittest.TestSuite):
 
     def _makeOrder(self, partition):
         """Return a order for the resource sets in partition."""
-        # This problem is NP-C - find the lowest cost hamiltonian path. It 
+        # This problem is NP-C - find the lowest cost hamiltonian path. It
         # also meets the triangle inequality, so we can use an approximation.
         # TODO: implement Christofides.
-        # See http://en.wikipedia.org/wiki/Travelling_salesman_problem#Metric_TSP
+        # See:
+        #   http://en.wikipedia.org/wiki/Travelling_salesman_problem#Metric_TSP
+
         # We need a root
         root = frozenset(['root'])
         partition.add(root)
@@ -361,9 +378,9 @@ class OptimisingTestSuite(unittest.TestSuite):
         node = root
         cycle = [node]
         steps = 2 * (len(mst) - 1)
-        for step in xrange(steps):
+        for step in range(steps):
             found = False
-            outgoing = None # For clearer debugging.
+            outgoing = None  # For clearer debugging.
             for outgoing in mst[node]:
                 if node in mst[outgoing]:
                     # we have a return path: take it
@@ -415,7 +432,7 @@ class TestResourceManager(object):
         dependencies list.
     :ivar resources: The resources that this resource needs. Calling
         neededResources will return the closure of this resource and its needed
-        resources. The resources list is in the same format as resources on a 
+        resources. The resources list is in the same format as resources on a
         test case - a list of tuples (attribute_name, resource).
     :ivar setUpCost: The relative cost to construct a resource of this type.
          One good approach is to set this to the number of seconds it normally
@@ -469,7 +486,8 @@ class TestResourceManager(object):
         the `clean` hook, which should do any resource-specific
         cleanup.
 
-        :param resource: A resource returned by `TestResource.getResource`.
+        :param resource: A resource returned by
+            `TestResourceManager.getResource`.
         :param result: An optional TestResult to report resource changes to.
         """
         self._uses -= 1
@@ -495,7 +513,7 @@ class TestResourceManager(object):
 
     def isDirty(self):
         """Return True if this managers cached resource is dirty.
-        
+
         Calling when the resource is not currently held has undefined
         behaviour.
         """
@@ -525,16 +543,17 @@ class TestResourceManager(object):
 
     def make(self, dependency_resources):
         """Override this to construct resources.
-        
+
         :param dependency_resources: A dict mapping name -> resource instance
             for the resources specified as dependencies.
+        :return: The made resource.
         """
         raise NotImplementedError(
             "Override make to construct resources.")
 
     def neededResources(self):
         """Return the resources needed for this resource, including self.
-        
+
         :return: A list of needed resources, in topological deepest-first
             order.
         """
@@ -550,36 +569,84 @@ class TestResourceManager(object):
         return result
 
     def reset(self, old_resource, result=None):
-        """Overridable method to return a clean version of old_resource.
+        """Return a clean version of old_resource.
 
         By default, the resource will be cleaned then remade if it had
-        previously been `dirtied`. 
+        previously been `dirtied` by the helper self._reset() - which is the
+        extension point folk should override to customise reset behaviour.
 
-        This function needs to take the dependent resource stack into
-        consideration as _make_all and _clean_all do.
+        This function takes the dependent resource stack into consideration as
+        _make_all and _clean_all do. The inconsistent naming is because reset
+        is part of the public interface, but _make_all and _clean_all is not.
 
-        :return: The new resource.
+        Note that if a resource A holds a lock or other blocking thing on
+        a dependency D, reset will result in this call sequence over a 
+        getResource(), dirty(), getResource(), finishedWith(), finishedWith()
+        sequence:
+        B.make(), A.make(), B.reset(), A.reset(), A.clean(), B.clean()
+        Thus it is important that B.reset not assume that A has been cleaned or
+        reset before B is reset: it should arrange to reference count, lazy
+        cleanup or forcibly reset resource in some fashion.
+
+        As an example, consider that B is a database with sample data, and
+        A is an application server serving content from it. B._reset() should
+        disconnect all database clients, reset the state of the database, and
+        A._reset() should tell the application server to dump any internal
+        caches it might have.
+
+        In principle we might make a richer API to allow before-and-after
+        reset actions, but so far that hasn't been needed.
+
+        :return: The possibly new resource.
         :param result: An optional TestResult to report resource changes to.
         """
-        if self._dirty:
-            self._clean_all(old_resource, result)
-            resource = self._make_all(result)
-        else:
-            resource = old_resource
+        # Core logic:
+        #  - if neither we nor any parent is dirty, do nothing.
+        # otherwise
+        #  - emit a signal to the test result
+        #  - reset all dependencies all, getting new attributes.
+        #  - call self._reset(old_resource, dependency_attributes)
+        #    [the default implementation does a clean + make]
+        if not self.isDirty():
+            return old_resource
+        self._call_result_method_if_exists(result, "startResetResource", self)
+        dependency_resources = {}
+        for name, mgr in self.resources:
+            dependency_resources[name] = mgr.reset(
+                getattr(old_resource, name), result)
+        resource = self._reset(old_resource, dependency_resources)
+        for name, value in dependency_resources.items():
+            setattr(resource, name, value)
+        self._call_result_method_if_exists(result, "stopResetResource", self)
         return resource
 
+    def _reset(self, resource, dependency_resources):
+        """Override this to reset resources other than via clean+make.
+
+        This method should reset the self._dirty flag (assuming the manager can
+        ever be clean) and return either the old resource cleaned or a fresh
+        one.
+
+        :param resource: The resource to reset.
+        :param dependency_resources: A dict mapping name -> resource instance
+            for the resources specified as dependencies.
+        """
+        self.clean(resource)
+        return self.make(dependency_resources)
+
     def _setResource(self, new_resource):
         """Set the current resource to a new value."""
         self._currentResource = new_resource
         self._dirty = False
 TestResource = TestResourceManager
 
-class GenericResource(TestResource):
-    """A TestResource that decorates an external helper of some kind.
 
-    GenericResource can be used to adapt an external resource so that 
+class GenericResource(TestResourceManager):
+    """A TestResourceManager that decorates an external helper of some kind.
+
+    GenericResource can be used to adapt an external resource so that
     testresources can use it. By default the setUp and tearDown methods are
-    called when making and cleaning the resource, and the resource is 
+    called when making and cleaning the resource, and the resource is
     considered permanently dirty, so it is torn down and brought up again
     between every use.
 
@@ -599,7 +666,7 @@ class GenericResource(TestResource):
         :param teardown_method_name: Optional method name to call to tear down
             the resource. Defaults to 'tearDown'.
         """
-        TestResource.__init__(self)
+        super(GenericResource, self).__init__()
         self.resource_factory = resource_factory
         self.setup_method_name = setup_method_name
         self.teardown_method_name = teardown_method_name
@@ -616,6 +683,52 @@ class GenericResource(TestResource):
         return True
 
 
+class FixtureResource(TestResourceManager):
+    """A TestResourceManager that decorates a ``fixtures.Fixture``.
+
+    The fixture has its setUp and cleanUp called as expected, and
+    reset is called between uses.
+
+    Due to the API of fixtures, dependency_resources are not
+    accessible to the wrapped fixture. However, if you are using
+    resource optimisation, you should wrap any dependencies in a
+    FixtureResource and set the resources attribute appropriately.
+    Note that when this is done, testresources will take care of
+    calling setUp and cleanUp on the dependency fixtures and so
+    the fixtures should not implicitly setUp or cleanUp their
+    dependencies (that have been mapped).
+
+    See the ``fixtures`` documentation for information on managing
+    dependencies within the ``fixtures`` API.
+
+    :ivar fixture: The wrapped fixture.
+    """
+
+    def __init__(self, fixture):
+        """Create a FixtureResource
+
+        :param fixture: The fixture to wrap.
+        """
+        super(FixtureResource, self).__init__()
+        self.fixture = fixture
+
+    def clean(self, resource):
+        resource.cleanUp()
+
+    def make(self, dependency_resources):
+        self.fixture.setUp()
+        return self.fixture
+
+    def _reset(self, resource, dependency_resources):
+        self.fixture.reset()
+        return self.fixture
+
+    def isDirty(self):
+        return True
+
+    _dirty = property(lambda _:True, lambda _, _1:None)
+
+
 class ResourcedTestCase(unittest.TestCase):
     """A TestCase parent or utility that enables cross-test resource usage.
 
@@ -627,14 +740,14 @@ class ResourcedTestCase(unittest.TestCase):
     from your setUp and tearDown (or whatever cleanup idiom is used).
 
     :ivar resources: A list of (name, resource) pairs, where 'resource' is a
-        subclass of `TestResource` and 'name' is the name of the attribute
-        that the resource should be stored on.
+        subclass of `TestResourceManager` and 'name' is the name of the
+        attribute that the resource should be stored on.
     """
 
     resources = []
 
     def setUp(self):
-        unittest.TestCase.setUp(self)
+        super(ResourcedTestCase, self).setUp()
         self.setUpResources()
 
     def setUpResources(self):
@@ -642,7 +755,7 @@ class ResourcedTestCase(unittest.TestCase):
 
     def tearDown(self):
         self.tearDownResources()
-        unittest.TestCase.tearDown(self)
+        super(ResourcedTestCase, self).tearDown()
 
     def tearDownResources(self):
         tearDownResources(self, self.resources, _get_result())
@@ -650,7 +763,7 @@ class ResourcedTestCase(unittest.TestCase):
 
 def setUpResources(test, resources, result):
     """Set up resources for test.
-    
+
     :param test: The test to setup resources for.
     :param resources: The resources to setup.
     :param result: A result object for tracing resource activity.
@@ -661,7 +774,7 @@ def setUpResources(test, resources, result):
 
 def tearDownResources(test, resources, result):
     """Tear down resources for test.
-    
+
     :param test: The test to tear down resources from.
     :param resources: The resources to tear down.
     :param result: A result object for tracing resource activity.
@@ -678,7 +791,7 @@ def _get_result():
     The result is passed to a run() or a __call__ method 4 or more frames
     up: that method is what calls setUp and tearDown, and they call their
     parent setUp etc. Its not guaranteed that the parameter to run will
-    be calls result as its not required to be a keyword parameter in 
+    be calls result as its not required to be a keyword parameter in
     TestCase. However, in practice, this works.
     """
     stack = inspect.stack()
@@ -690,4 +803,3 @@ def _get_result():
             if (result is not None and
                 getattr(result, 'startTest', None) is not None):
                 return result
-
